﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MirrorMatch
{
	class BezierShape : IShape
	{
		private Vector2[] v;
        private List<LineShape> lineList;
        private BezierShape[] splitCurves;
        private ConvexHull2D4 hull;
        
        //debuginfo
        private List<LineShape> debugLines = new List<LineShape>();

		public BezierShape(Vector2 v0, Vector2 v1, Vector2 v2, Vector2 v3)
		{
			v = new Vector2[4];
			v[0] = v0;
			v[1] = v1;
			v[2] = v2;
			v[3] = v3;
            lineList = getLineList(30);
		}


        public void texturize(Texture2D texture)
        {
            foreach (LineShape line in lineList)
            {
                line.texturize(texture);
            }
        }

        public void draw(SpriteBatch spriteBatch)
        {
            foreach (LineShape line in lineList)
            {
                line.draw(spriteBatch);
            }
            foreach (LineShape line in debugLines)
            {
                line.texturize(ResourceManager.Instance.GetTexture("default_line"));
                line.draw(spriteBatch);
            }
        }

		private List<LineShape> getLineList(int nrOfEdges)
		{
			float dt = 1f / (float)nrOfEdges;
		 
			List<Vector2> pointList = new List<Vector2>();
			Vector2 c = new Vector2(3*(v[1].X - v[0].X), 3*(v[1].Y - v[0].Y));
			Vector2 b = new Vector2(3*(v[2].X - v[1].X) - c.X, 3*(v[2].Y - v[1].Y) - c.Y);
			Vector2 a = new Vector2(v[3].X - v[0].X - c.X - b.X, v[3].Y - v[0].Y - c.Y - b.Y);

			//Calculate all points
			for (int i = 0; i < nrOfEdges; i++)
			{
				float t = dt * i;
				float x = a.X * (float)Math.Pow((double)t, 3.0) + b.X * (float)Math.Pow((double)t, 2.0) + c.X * t + v[0].X;
				float y = a.Y * (float)Math.Pow((double)t, 3.0) + b.Y * (float)Math.Pow((double)t, 2.0) + c.Y * t + v[0].Y;
				pointList.Add(new Vector2(x, y));
			}

			//Create lines out of the points
			List<LineShape> lineList = new List<LineShape>();
			for (int i = 0; i < nrOfEdges; i++)
			{
				if (i == (nrOfEdges - 1))
				{
					lineList.Add(new LineShape(pointList.ElementAt(i), v[3]));
				}
				else
				{
					lineList.Add(new LineShape(pointList.ElementAt(i), pointList.ElementAt(i + 1)));
				}
			}

			return lineList;
		}

        public ConvexHull2D4 convexHull()
        {
            if(hull == null)
                hull = new ConvexHull2D4(v[0], v[1], v[2], v[3]);
            return hull;
        }


        public Vector2[] getBoundingVolume()
        {
            return new Vector2[]
            {
                v[0],
                v[1],
                v[2],
                v[3]
            };
        }

        public RayIntersection intersects(Ray ray)
        {

            //debugLines = convexHull().LineList;

            if (RayConvexHull2D4Overlap(ray, convexHull()))
            {
                // Split the bezier curve till the ray misses OR till the maximum divide limit is reached
                BezierShape[] possibleHits = split();
                int divideLimit = 11;	/*This may be changed to alter the precision...
                                          ...The normal offset can also be calibrated after this is changed.
                                          DivideLimit's maximum value seems to be around 10
                                          so we don't get a jump-back effect.*/
                RayIntersection hitData = new RayIntersection();
                hitData.T = float.MaxValue;
                if (goDeeper(ray, possibleHits, divideLimit - 1, hitData))
                {
                    return hitData;
                }
            }

            return null;
        }

        private bool goDeeper(Ray ray, BezierShape[] possibleHits, int divideLimit, RayIntersection hitData)
        {
            bool intersects = false;

            if (divideLimit > 0)
            {
                for (int i = 0; i < possibleHits.Count(); i++)
                {
                    if (RayConvexHull2D4Overlap(ray, possibleHits[i].convexHull()))
                    {
                        //if none of the possible hits overlaps, there is no intersection, intersects variable will be false
                        if (goDeeper(ray, possibleHits[i].split(), divideLimit - 1, hitData))
                        {
                            intersects = true; //if any recursive sub-step returned true, there is an intersection
                        }
                    }
                }
            }
            else if (divideLimit == 0)
            {
                Vector2 hitPosition = (possibleHits[0].v[0] + possibleHits[0].v[1] + possibleHits[0].v[2] + possibleHits[0].v[3])/4f ;
                if ((hitPosition - ray.Origin).Length() < hitData.T)
                {
                    LineShape line = new LineShape(possibleHits[0].v[0], possibleHits[0].v[3]);
                    Vector2 tmpDirection = line.direction();
                    hitData.Normal = new Vector2(tmpDirection.Y, -tmpDirection.X);
                    
                    hitData.Normal.Normalize();
                    hitData.T = (hitPosition - ray.Origin).Length();
                }
                intersects = true;
            }

            return intersects;
        }

        //private bool RayAABB2dOverlap(Ray ray, AABB2D box)
        //{
        //    float t, s;
        //    bool hit = false;

        //    foreach (LineSegment line in box.LineList)
        //    {
        //        if (!hit)
        //        {
        //            Vector2 tmpDirection = line.destination - line.origin;
        //            if (Vector2.Dot(new Vector2(tmpDirection.Y, -tmpDirection.X), ray.direction) != 0)
        //            {
        //                // Get the t value of ray. (t > 0)
        //                t = Vector2.Dot(new Vector2(tmpDirection.Y, -tmpDirection.X), line.origin - ray.originPosition)
        //                    / Vector2.Dot(new Vector2(tmpDirection.Y, -tmpDirection.X), ray.direction);

        //                // Get the s value of line segment. (0 < t < 1)
        //                s = Vector2.Dot(new Vector2(ray.direction.Y, -ray.direction.X), line.origin - ray.originPosition)
        //                    / Vector2.Dot(new Vector2(tmpDirection.Y, -tmpDirection.X), ray.direction);

        //                if (t > 0 && s >= 0 && s <= 1)
        //                {
        //                    hit = true;
        //                }
        //            }
        //        }
        //    }

        //    return hit;
        //}

        private bool RayConvexHull2D4Overlap(Ray ray, ConvexHull2D4 hull)
        {
            float t, s;

            foreach (LineShape line in hull.LineList)
            {
                Vector2 tmpDirection = line.Destination - line.Origin;
                if (Vector2.Dot(new Vector2(tmpDirection.Y, -tmpDirection.X), ray.Direction) != 0)
                {
                    // Get the t value of ray. (t > 0)
                    t = Vector2.Dot(new Vector2(tmpDirection.Y, -tmpDirection.X), line.Origin - ray.Origin)
                        / Vector2.Dot(new Vector2(tmpDirection.Y, -tmpDirection.X), ray.Direction);

                    // Get the s value of line segment. (0 < t < 1)
                    s = Vector2.Dot(new Vector2(ray.Direction.Y, -ray.Direction.X), line.Origin - ray.Origin)
                        / Vector2.Dot(new Vector2(tmpDirection.Y, -tmpDirection.X), ray.Direction);

                    if (t > 0 && s >= -0.001 && s <= 1.001)
                    {
                        return true;
                    }
                }
            }
            return false;
        }
        //private Vector2[] V
        //{
        //    get
        //    {
        //        return v;
        //    }
        //}

		private BezierShape[] split()
		{
            if (splitCurves == null)
            {
                // Level 1
                LineShape[] level_one = new LineShape[3];
                level_one[0] = new LineShape(v[0], v[1]);
                level_one[1] = new LineShape(v[1], v[2]);
                level_one[2] = new LineShape(v[2], v[3]);

                // Level 2
                LineShape[] level_two = new LineShape[2];
                level_two[0] = new LineShape(level_one[0].Center, level_one[1].Center);
                level_two[1] = new LineShape(level_one[1].Center, level_one[2].Center);

                // Level 3
                LineShape level_three = new LineShape(level_two[0].Center, level_two[1].Center);
                Vector2 mid_point = level_three.Center;

                // Create two new curves
                splitCurves = new BezierShape[2];
                splitCurves[0] = new BezierShape(v[0], level_one[0].Center, level_two[0].Center, mid_point);
                splitCurves[1] = new BezierShape(mid_point, level_two[1].Center, level_one[2].Center, v[3]);
            }

            return splitCurves;
		}
	}
}