﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MirrorMatch
{
    //kD-tree with entities in leafs only.
    //The whole tree is pre-generated with a constant depth.
    //Entities are added to all leafs they overlap.
    class KdTree
    {
        public const bool HORIZONTAL = false;
        public const bool VERTICAL = true;

        Vector2 min, max;
        KdTree[] children = new KdTree[2];
        List<Entity> entities;
        bool direction;

        public static bool debugColoring = false;
        private int debugColor = 0;

        public static KdTree generate(Vector2 min, Vector2 max, int depth, bool direction)
        {
            if (depth <= 0)
                return null;

            KdTree node = new KdTree(min, max);
            node.direction = direction;

            if (direction == VERTICAL) //vertical
            {
                node.children[0] = generate(new Vector2(min.X, min.Y), new Vector2(max.X, (min.Y + max.Y)/2), depth - 1, !direction);
                node.children[1] = generate(new Vector2(min.X, (min.Y + max.Y)/2), new Vector2(max.X, max.Y), depth - 1, !direction);
            }
            else //horizontal
            {
                node.children[0] = generate(new Vector2(min.X, min.Y), new Vector2((min.X + max.X)/2, max.Y), depth - 1, !direction);
                node.children[1] = generate(new Vector2((min.X + max.X) / 2, min.Y), new Vector2(max.X, max.Y), depth - 1, !direction);
            }

            return node;
        }

        public void addEntity(Entity entity)
        {
            Vector2[] corners = entity.getBoundingVolume();
            bool allOutLeft = true;
            bool allOutRight = true;
            bool allOutTop = true;
            bool allOutBottom = true;

            foreach (Vector2 corner in corners)
            {
                if (corner.X <= max.X)
                    allOutLeft = false;
                if (corner.X >= min.X)
                    allOutRight = false;
                if (corner.Y <= max.Y)
                    allOutBottom = false;
                if (corner.Y >= min.Y)
                    allOutTop = false;
            }

            //the entity lies outside of this node if all corners are outside on the SAME side
            if (allOutLeft || allOutRight || allOutTop || allOutBottom)
                return;

            if (children[0] == null && children[1] == null)
            {
                //add to self if this is a leaf
                entities.Add(entity);
            }
            else
            {
                //try adding to children if this is a non-leaf
                children[0].addEntity(entity);
                children[1].addEntity(entity);
            }
        }

        //the distance is the distance to this node along the ray
        //the value is first calculated for both children BEFORE recursing to them
        //the same value is then used by that children AFTER recursing.
        //the value is passed as a parameter to avoid re-calculation
        public RayIntersection getClosestIntersection(Ray ray, float maxDistance = float.MaxValue)
        {
            // Set debugColoring to true before the intersection test, and false after, 
            // to enable the debug coloring for only those intersections

            //debug color of this node is red if the node is checked for intersection, but is distance culled or not intersecting
            if (debugColoring)
                debugColor = 1;

            //if ray not intersecting this node, return null
            if (
                new LineShape(new Vector2(min.X, min.Y), new Vector2(max.X, min.Y)).intersects(ray) == null &&
                new LineShape(new Vector2(min.X, min.Y), new Vector2(min.X, max.Y)).intersects(ray) == null &&
                new LineShape(new Vector2(min.X, max.Y), new Vector2(max.X, max.Y)).intersects(ray) == null &&
                new LineShape(new Vector2(max.X, min.Y), new Vector2(max.X, max.Y)).intersects(ray) == null)
            {
                return null;
            }

            //debug color of this node is green if the ray is intersecting
            if (debugColoring)
                debugColor = 2;

            //if we're not in a leaf, enter the closest child first.
            if (children[0] != null && children[1] != null)
            {
                //calculate distances and enter the closest child first
                float distance0 = Vector2.Dot((children[0].min + children[0].max) / 2 - ray.Origin, ray.Direction);
                float distance1 = Vector2.Dot((children[1].min + children[1].max) / 2 - ray.Origin, ray.Direction);

                KdTree closeChild;
                KdTree farChild;
                float closeDistance;
                float farDistance;
                if (distance0 < distance1)
                {
                    closeChild = children[0];
                    farChild = children[1];
                    closeDistance = distance0;
                    farDistance = distance1;
                }
                else
                {
                    closeChild = children[1];
                    farChild = children[0];
                    closeDistance = distance1;
                    farDistance = distance0;
                }


                //A node will be culled if the whole node is further away than the closest found intersection.
                //The earlier we find an intersection near the ray origin, the more nodes will be culled.
                //Therefore we step into closest child first.
                if (distance0 < distance1)
                {
                    //enter child 0 first (closest child)
                    RayIntersection returnValue = children[0].getClosestIntersection(ray);
                    if (returnValue != null)
                        return returnValue; //if not null, it's the closest intersection since we always step into closest child first
                    returnValue = children[1].getClosestIntersection(ray);
                    if (returnValue != null)
                        return returnValue; //must be non-null, since we know that this node is intersecting and the other child isn't
                    return null; //No intersection in any children
                }
                else
                {
                    //enter child 1 first (closest child)
                    RayIntersection returnValue = children[1].getClosestIntersection(ray);
                    if (returnValue != null)
                        return returnValue; //if not null, it's the closest intersection since we always step into closest child first
                    returnValue = children[0].getClosestIntersection(ray);
                    if (returnValue != null)
                        return returnValue; //must be non-null, since we know that this node is intersecting and the other child isn't
                    return null;
                }
            }
            else
            {
                //we're in a leaf, test intersection against all children and return the closest
                RayIntersection closestHit = null; //null returned if no intersection
                foreach (Entity entity in entities)
                {
                    RayIntersection hit = entity.intersects(ray);
                    if (hit != null)
                    {
                        if (closestHit == null || hit.T < closestHit.T)
                        {
                            //it's the closest intersection

                            //Since entities are added to all leafs they overlap
                            //it is possible that the intersection point is not inside this node.
                            //Only accept intersections that occur INSIDE this node!
                            //Otherwise eternal chaos shall torment mankind.
                            Vector2 hitPoint = ray.Origin + ray.Direction * hit.T;
                            if (
                                hitPoint.X >= min.X - 0.01f &&
                                hitPoint.X <= max.X + 0.01f &&
                                hitPoint.Y >= min.Y - 0.01f &&
                                hitPoint.Y <= max.Y + 0.01f)
                            {
                                //the intersection is a valid intersection, and also the closest so far
                                closestHit = hit;
                            }
                        }
                    }
                }
                return closestHit;
            }
        }

        public void drawDebugData(SpriteBatch spriteBatch, int offset = 0)
        {
            //the offset makes it easier to distinguish different levels of the tree in the debug data
            Color color = Color.White;

            //debug
            if (debugColor == 1)
                color = Color.Red;
            else if (debugColor == 2)
                color = Color.Green;

            //debug
            debugColor = 0;

            Texture2D line = ResourceManager.Instance.GetTexture("default_line");
            int xMin = (int)min.X + offset;
            int xMax = (int)max.X - offset;
            int yMin = (int)min.Y + offset;
            int yMax = (int)max.Y - offset;
            int sizeY = yMax - yMin;
            int sizeX = xMax - xMin;

            spriteBatch.Draw(line, new Rectangle(xMin, yMin, 1, sizeY), color);
            spriteBatch.Draw(line, new Rectangle(xMin, yMin, sizeX, 1), color);
            spriteBatch.Draw(line, new Rectangle(xMax, yMin, 1, sizeY), color);
            spriteBatch.Draw(line, new Rectangle(xMin, yMax, sizeX, 1), color);

            foreach (KdTree child in children)
                if (child != null)
                    child.drawDebugData(spriteBatch, offset + 2);
        }

        public KdTree(Vector2 min, Vector2 max)
        {
            this.min = min;
            this.max = max;

            for (int i = 0; i < 2; i++)
            {
                children[i] = null;
            }

            entities = new List<Entity>();
        }
    }

}
