﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MirrorMatch
{
    class ScoreData
    {
        #region FIELDS
        // All properties should have backing fields
        Dictionary<string, float> scoreBoard;
        Dictionary<string, int> victoryBoard;
        int victoryLimit;
        #endregion

        #region CONSTRUCTORS
        public ScoreData()
        {
            scoreBoard = new Dictionary<string, float>();
            victoryBoard = new Dictionary<string, int>();
        }

        public ScoreData(params string[] playerIDs): this()
        {
            foreach (string player in playerIDs)
            {
                scoreBoard.Add(player, 0);
                victoryBoard.Add(player, 0);
            }
        }
        #endregion

        #region WITH SIDE EFFECTS
        /// <summary>
        /// Increase (or decrease) the score for the indicated player. If the player does not exist it will be created
        /// </summary>
        /// <param name="playerID">The ID of the player that scored</param>
        /// <param name="score">A positive or negative float indicating a change in score</param>
        public void RegisterScore(string playerID, float score)
        {
            if (scoreBoard.ContainsKey(playerID))
                scoreBoard[playerID] += score;
            else
                scoreBoard.Add(playerID, score);
        }
        /// <summary>
        /// Increace number of victories by one
        /// </summary>
        /// <param name="playerID">ID of the player that won</param>
        public void RegisterVictory(string playerID)
        {
            if (victoryBoard.ContainsKey(playerID))
                victoryBoard[playerID] += 1;
            else
                victoryBoard.Add(playerID, 1);
        }
        /// <summary>
        /// When the victory limit is reached a player has won
        /// </summary>
        public int VictoryLimit
        {
            get { return victoryLimit; }
            set 
            {
                if (value > 0)
                    victoryLimit = value;
                else
                    throw new ArgumentOutOfRangeException("Victory limit must be larger than zero");
            }
        }
        #endregion

        #region NO SIDE EFFECTS
        public Dictionary<string, float> ScoreBoard
        {
            get { return scoreBoard; }
        }
        public Dictionary<string, int> VictoryBoard
        {
            get { return victoryBoard; }
        }
        /// <summary>
        /// Print 
        /// </summary>
        public string ScoreBoardAsString
        {
            get
            { 
                string result="";
                KeyValuePair<string, float> row;
                for(int i=0; i<scoreBoard.Count; i++)
                {
                    row = scoreBoard.ElementAt<KeyValuePair<string, float>>(i);
                    result += row.Key + ": " + row.Value + " points\n";
                }
                return result;
            }
        }
        public string VictoryBoardAsString
        {
            get
            {
                string result = "";
                KeyValuePair<string, int> row;
                for (int i = 0; i < victoryBoard.Count; i++)
                {
                    row = victoryBoard.ElementAt<KeyValuePair<string, int>>(i);
                    result += row.Key + ": " + row.Value + " points\n";
                }
                return result;
            }
        }
        /// <summary>
        /// Return the ID of the player who has the highest score
        /// </summary>
        public string HighestScore
        {
            get
            {
                KeyValuePair<string, float> leader = scoreBoard.ElementAt<KeyValuePair<string, float>>(0);
                foreach (KeyValuePair<string, float> row in scoreBoard)
                {
                    if (row.Value > leader.Value)
                        leader = row;
                }
                return leader.Key;
            }
        }
        /// <summary>
        /// Return the ID of the player who has won the most victories
        /// </summary>
        public string HighestNumberOfVictories
        {
            get
            {
                // Get the first registered player
                KeyValuePair<string, int> leader = victoryBoard.ElementAt<KeyValuePair<string, int>>(0);
                // Loop through all players to see if anyone have a higher number of victories
                foreach (KeyValuePair<string, int> row in victoryBoard)
                {
                    if (row.Value > leader.Value)
                        leader = row;
                }
                return leader.Key;
            }
        }
        public bool PlayerHasWon
        {
            get 
            {
                string leader = HighestNumberOfVictories;
                if (victoryBoard[leader] >= victoryLimit)
                    return true;
                else
                    return false;
            }
        }
        #endregion
    }
}
