﻿/*Class not created by us*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

class XnaTextField
{
    private String textHolder;
    private Rectangle bounds, borderBounds;
    private KeyboardState keyboardState, prevState;
    private List<Keys> specialKeyList;
    private Keys keyModifier;
    private SpriteFont font;
    private Color textColor, borderColor, foreGroundColor;
    private Texture2D texture;
    private ContentManager content;
    private Point textPadding;
    private int borderSize;
    private GraphicsDevice graphicsDevice;
    private bool caretIsVisible, caretIsActive;
    public bool IsActive { get; set; }
        
    /* BEGIN Constructor section */
    private XnaTextField()
    {
        IsActive = true;
        int x = 20, y = 20, width = 200, height = 30;
        this.textHolder = "";
        this.borderSize = 4;
        this.bounds = new Rectangle(x, y, width, height);
        this.borderBounds = new Rectangle(bounds.X - borderSize / 2, bounds.Y - borderSize / 2, bounds.Width + borderSize, bounds.Height + borderSize);
        this.textColor = Color.Black;
        this.foreGroundColor = Color.White;
        this.borderColor = Color.Black;
        this.specialKeyList = new List<Keys>();
        this.textPadding = new Point(3, 3);
        this.caretIsActive = true;
        this.InitDefaultSpecialKeys();
            
    }

    public XnaTextField(String font, ContentManager content, GraphicsDevice graphicsDevice)
        : this()
    {
        Color[] color = new Color[1];
        color[0] = Color.White;
        this.content = content;
        this.font = this.content.Load<SpriteFont>(font);
        this.texture = new Texture2D(graphicsDevice, 1, 1);
        this.texture.SetData<Color>(color);
        this.graphicsDevice = graphicsDevice;
    }
    /* END Constructor section */


    /* BEGIN Private methods */
    private void InitDefaultSpecialKeys()
    {
        this.AddSpecialKey(Keys.Space);
        this.AddSpecialKey(Keys.LeftShift);
        this.AddSpecialKey(Keys.Back);
        this.AddSpecialKey(Keys.OemTilde);
        this.AddSpecialKey(Keys.OemQuotes);
        this.AddSpecialKey(Keys.OemCloseBrackets);
        this.AddSpecialKey(Keys.CapsLock);
        this.AddSpecialKey(Keys.OemPeriod);
    }

    private String GetKeyAsString(Keys key)
    {
        String temp = key.ToString();
            
        if(this.specialKeyList.Contains(key))
            temp="";

        if ((int)key >= 48 && (int)key <= 57)
            temp = temp.Replace("D", "");

        switch (key)
        {
            case Keys.Space:
                temp = " ";
                break;
            case Keys.OemCloseBrackets:
                temp = "å";
                break;
            case Keys.OemQuotes:
                temp = "ä";
                break;
            case Keys.OemTilde:
                temp = "ö";
                break;
            case Keys.OemPeriod:
                temp = ".";
                break;
            case Keys.LeftShift:
                this.keyModifier = key;
                break;
            case Keys.CapsLock:
                if (this.keyModifier != Keys.CapsLock)
                    this.keyModifier = key;
                else
                    this.keyModifier = Keys.None;
                break;
        }
        return temp;
    }

    private void AddSpecialKey(Keys key)
    {
        this.specialKeyList.Add(key);
    }
    /* END Private Methods */
              

    /* BEGIN Public methods */
        
    /// <summary>
    /// Clears the textfield from all text
    /// </summary>
    public void Clear()
    {
        this.textHolder = "";
    }

    /// <summary>
    /// Allows finetuning of the textposition.
    /// </summary>
    /// <param name="x">How much additional space on the left side to add</param>
    /// <param name="y">How much additional space on the top to add</param>
    public void SetTextPadding(int x, int y)
    {
        this.textPadding.X = x;
        this.textPadding.Y = y;
    }

    /// <summary>
    /// Fills the textfield with some initial text.
    /// </summary>
    /// <param name="text">The text to add to the the field</param>
    public void SetText(String text)
    {
        this.textHolder = text;
    }

    /// <summary>
    /// Sets the color of the border.
    /// </summary>
    /// <param name="borderColor">The new border color</param>
    public void SetBorderColor(Color borderColor)
    {
        this.borderColor = borderColor;
    }

    /// <summary>
    /// Sets the color of the foreround. 
    /// </summary>
    /// <param name="foreGroundColor">The new foreground color</param>
    public void SetForeGroundColor(Color foreGroundColor)
    {
        this.foreGroundColor = foreGroundColor;
    }

    /// <summary>
    /// Sets the color of the text.
    /// </summary>
    /// <param name="textColor">The new text color</param>
    public void SetTextColor(Color textColor)
    {
        this.textColor = textColor;
    }

    /// <summary>
    /// Sets the font, note that this could be changed anytime during runtime.
    /// </summary>
    /// <param name="font">The name of the spritefont</param>
    public void SetFont(String font)
    {
        this.font = content.Load<SpriteFont>(font);
    }

    /// <summary>
    /// Sets the size of the border.
    /// </summary>
    /// <param name="px">How thick the border should be in pixels</param>
    public void SetBorderSize(int px)
    {
        this.borderSize = px;
        this.borderBounds.X -= this.borderSize / 2;
        this.borderBounds.Y -= this.borderSize / 2;
        this.borderBounds.Width = this.borderBounds.Width + this.borderSize;
        this.borderBounds.Height = this.borderBounds.Height + this.borderSize;
    }

    /// <summary>
    /// Sets the position, of the textfield, on the screen where 0,0 is the upper left corner.
    /// </summary>
    /// <param name="x">Where on the X-axis the textfield should be positioned</param>
    /// <param name="y">Where on the Y-axis the textfield should be positioned</param>
    public void SetPosition(int x, int y)
    {
        this.bounds.X = x;
        this.bounds.Y = y;
        this.borderBounds.X = this.bounds.X - this.borderSize / 2;
        this.borderBounds.Y = this.bounds.Y - this.borderSize / 2;
        this.borderBounds.Width = this.bounds.Width + this.borderSize;
        this.borderBounds.Height = this.bounds.Height + this.borderSize;
    }
        
    /// <summary>
    /// Sets the size for the textfield.
    /// </summary>
    public void SetSize(int width, int height)
    {
        this.bounds.Width = width;
        this.bounds.Height = height;

        this.borderBounds.Width = this.bounds.Width + this.borderSize;
        this.borderBounds.Height = this.bounds.Height + this.borderSize;
    }


    /// <summary>
    /// Returns the bounding rectangle for the textfield. 
    /// Useful if you want to check collision against it.
    /// </summary>
    public Rectangle GetBounds()
    {
        return borderBounds;
    }

    /// <summary>
    /// Gets the current text from the textfield.
    /// </summary>
    public String GetText()
    {
        return this.textHolder;
    }

    /// <summary>
    /// Allows the user to disable the caret.
    /// </summary>
    /// <param name="isEnabled">Disable or enable the caret</param>
    public void CaretEnabled(bool isEnabled)
    {
        this.caretIsActive = isEnabled;    
    }

    /// <summary>
    /// Allows the textfield to run logic such as scanning the keyboard.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    public void Update(GameTime gameTime)
    {
        if (this.IsActive)
        {
            this.keyboardState = Keyboard.GetState();
            foreach (Keys key in keyboardState.GetPressedKeys())
            {
                if ((this.prevState.IsKeyUp(key) && (((int)key >= 65 && (int)key <= 90) || ((int)key >= 48 && (int)key <= 57))) || (this.prevState.IsKeyUp(key) && this.specialKeyList.Contains(key)))
                {
                    if (key == Keys.Back)
                    {
                        if (this.textHolder.Length >= 1)
                            this.textHolder = this.textHolder.Remove(this.textHolder.Length - 1);
                    }
                    if (this.keyModifier != Keys.CapsLock)
                    {
                        if (this.keyModifier == Keys.LeftShift)
                        {
                            if (this.font.MeasureString(this.textHolder).X + this.textPadding.X < this.bounds.Width - 5 - this.borderSize)
                                this.textHolder += this.GetKeyAsString(key).ToUpper();
                            this.keyModifier = Keys.None;
                        }
                        else
                        {
                            if (this.font.MeasureString(this.textHolder).X + this.textPadding.X < this.bounds.Width - 5 - this.borderSize)
                                this.textHolder += this.GetKeyAsString(key).ToLower();
                        }
                    }
                    else
                    {
                        if (this.font.MeasureString(this.textHolder).X + this.textPadding.X < this.bounds.Width - 5 - this.borderSize)
                            this.textHolder += this.GetKeyAsString(key).ToUpper();
                    }
                }
            }

            this.prevState = this.keyboardState;

            if (gameTime.TotalGameTime.Milliseconds % 500 == 0)
            {
                if (this.caretIsVisible)
                    this.caretIsVisible = false;
                else
                    this.caretIsVisible = true;
            }
        }
    }

    public void Draw(SpriteBatch spriteBatch)
    {
        spriteBatch.Draw(this.texture, this.borderBounds, borderColor);
        spriteBatch.Draw(this.texture, this.bounds, foreGroundColor);
        spriteBatch.DrawString(this.font, this.textHolder, new Vector2(this.bounds.X + this.textPadding.X, this.bounds.Y + this.textPadding.Y), this.textColor);
        if (this.caretIsActive)
        {
            if (this.caretIsVisible)
                spriteBatch.DrawString(this.font, "|", new Vector2(this.bounds.X + this.textPadding.X + this.font.MeasureString(textHolder).X, this.bounds.Y + this.textPadding.Y), this.textColor);
        }
    }
    
    /* END Public methods */
}